@@ -1205,20 +1205,31 @@ static int smtp_loop(SMTP_STATE *state, NOCLOBBER int send_state,
 	     * information, the command length stays within the 512 byte
 	     * command line length limit.
 	     */
+#ifndef CAN_FORWARD_CLIENT_NAME
+#define _ATTR_AVAIL_AND_KNOWN_(val) \
+	(DEL_REQ_ATTR_AVAIL(val) && strcasecmp((val), "unknown"))
+#define CAN_FORWARD_CLIENT_NAME	_ATTR_AVAIL_AND_KNOWN_
+#define CAN_FORWARD_CLIENT_ADDR	_ATTR_AVAIL_AND_KNOWN_
+#define CAN_FORWARD_CLIENT_PORT	_ATTR_AVAIL_AND_KNOWN_
+#define CAN_FORWARD_PROTO_NAME	_ATTR_AVAIL_AND_KNOWN_
+#define CAN_FORWARD_HELO_NAME	DEL_REQ_ATTR_AVAIL
+#define CAN_FORWARD_RWR_CONTEXT	DEL_REQ_ATTR_AVAIL
+#endif
+
 	case SMTP_STATE_XFORWARD_NAME_ADDR:
 	    vstring_strcpy(next_command, XFORWARD_CMD);
 	    if ((session->features & SMTP_FEATURE_XFORWARD_NAME)
-		&& DEL_REQ_ATTR_AVAIL(request->client_name)) {
+		&& CAN_FORWARD_CLIENT_NAME(request->client_name)) {
 		vstring_strcat(next_command, " " XFORWARD_NAME "=");
 		xtext_quote_append(next_command, request->client_name, "");
 	    }
 	    if ((session->features & SMTP_FEATURE_XFORWARD_ADDR)
-		&& DEL_REQ_ATTR_AVAIL(request->client_addr)) {
+		&& CAN_FORWARD_CLIENT_ADDR(request->client_addr)) {
 		vstring_strcat(next_command, " " XFORWARD_ADDR "=");
 		xtext_quote_append(next_command, request->client_addr, "");
 	    }
 	    if ((session->features & SMTP_FEATURE_XFORWARD_PORT)
-		&& DEL_REQ_ATTR_AVAIL(request->client_port)) {
+		&& CAN_FORWARD_CLIENT_PORT(request->client_port)) {
 		vstring_strcat(next_command, " " XFORWARD_PORT "=");
 		xtext_quote_append(next_command, request->client_port, "");
 	    }
@@ -1231,17 +1242,17 @@ static int smtp_loop(SMTP_STATE *state, NOCLOBBER int send_state,
 	case SMTP_STATE_XFORWARD_PROTO_HELO:
 	    vstring_strcpy(next_command, XFORWARD_CMD);
 	    if ((session->features & SMTP_FEATURE_XFORWARD_PROTO)
-		&& DEL_REQ_ATTR_AVAIL(request->client_proto)) {
+		&& CAN_FORWARD_PROTO_NAME(request->client_proto)) {
 		vstring_strcat(next_command, " " XFORWARD_PROTO "=");
 		xtext_quote_append(next_command, request->client_proto, "");
 	    }
 	    if ((session->features & SMTP_FEATURE_XFORWARD_HELO)
-		&& DEL_REQ_ATTR_AVAIL(request->client_helo)) {
+		&& CAN_FORWARD_HELO_NAME(request->client_helo)) {
 		vstring_strcat(next_command, " " XFORWARD_HELO "=");
 		xtext_quote_append(next_command, request->client_helo, "");
 	    }
 	    if ((session->features & SMTP_FEATURE_XFORWARD_DOMAIN)
-		&& DEL_REQ_ATTR_AVAIL(request->rewrite_context)) {
+		&& CAN_FORWARD_RWR_CONTEXT(request->rewrite_context)) {
 		vstring_strcat(next_command, " " XFORWARD_DOMAIN "=");
 		xtext_quote_append(next_command,
 		     strcmp(request->rewrite_context, MAIL_ATTR_RWR_LOCAL) ?
@@ -1923,19 +1934,19 @@ int     smtp_xfer(SMTP_STATE *state)
     send_name_addr =
 	var_smtp_send_xforward
 	&& (((session->features & SMTP_FEATURE_XFORWARD_NAME)
-	     && DEL_REQ_ATTR_AVAIL(request->client_name))
+	     && CAN_FORWARD_CLIENT_NAME(request->client_name))
 	    || ((session->features & SMTP_FEATURE_XFORWARD_ADDR)
-		&& DEL_REQ_ATTR_AVAIL(request->client_addr))
+		&& CAN_FORWARD_CLIENT_ADDR(request->client_addr))
 	    || ((session->features & SMTP_FEATURE_XFORWARD_PORT)
-		&& DEL_REQ_ATTR_AVAIL(request->client_port)));
+		&& CAN_FORWARD_CLIENT_PORT(request->client_port)));
     session->send_proto_helo =
 	var_smtp_send_xforward
 	&& (((session->features & SMTP_FEATURE_XFORWARD_PROTO)
-	     && DEL_REQ_ATTR_AVAIL(request->client_proto))
+	     && CAN_FORWARD_PROTO_NAME(request->client_proto))
 	    || ((session->features & SMTP_FEATURE_XFORWARD_HELO)
-		&& DEL_REQ_ATTR_AVAIL(request->client_helo))
+		&& CAN_FORWARD_HELO_NAME(request->client_helo))
 	    || ((session->features & SMTP_FEATURE_XFORWARD_DOMAIN)
-		&& DEL_REQ_ATTR_AVAIL(request->rewrite_context)));
+		&& CAN_FORWARD_RWR_CONTEXT(request->rewrite_context)));
     if (send_name_addr)
 	recv_state = send_state = SMTP_STATE_XFORWARD_NAME_ADDR;
     else if (session->send_proto_helo)
